@@ -14,7 +14,7 @@
 // You should have received a copy of the GNU General Public License
 // along with the snarkOS library. If not, see <https://www.gnu.org/licenses/>.
 
-use crate::{errors::NetworkError, message::*, ConnReader, ConnWriter, Node, Receiver, Sender, State};
+use crate::{errors::NetworkError, message::*, stats, Cache, ConnReader, ConnWriter, Node, Receiver, Sender, State};
 
 use std::{collections::HashMap, net::SocketAddr, sync::Arc, time::Duration};
 
@@ -23,7 +23,7 @@ use snarkvm_objects::Storage;
 use tokio::{
     io::{AsyncReadExt, AsyncWriteExt},
     net::{TcpListener, TcpStream},
-    sync::mpsc::channel,
+    sync::mpsc::{channel, error::TrySendError},
     task,
 };
 
@@ -42,7 +42,7 @@ pub struct Inbound {
 impl Default for Inbound {
     fn default() -> Self {
         // Initialize the sender and receiver.
-        let (sender, receiver) = tokio::sync::mpsc::channel(64 * 1024);
+        let (sender, receiver) = tokio::sync::mpsc::channel(crate::INBOUND_CHANNEL_DEPTH);
 
         Self {
             sender,
@@ -52,13 +52,6 @@ impl Default for Inbound {
 }
 
 impl Inbound {
-    #[inline]
-    pub(crate) async fn route(&self, response: Message) {
-        if let Err(err) = self.sender.send(response).await {
-            error!("Failed to route a response for a message: {}", err);
-        }
-    }
-
     #[inline]
     pub(crate) fn take_receiver(&self) -> Receiver {
         self.receiver
@@ -71,91 +64,91 @@ impl Inbound {
 impl<S: Storage + Send + Sync + 'static> Node<S> {
     /// This method handles new inbound connection requests.
     pub async fn listen(&self) -> Result<(), NetworkError> {
-        let (listener_address, listener) = if let Some(addr) = self.config.desired_address {
-            let listener = TcpListener::bind(&addr).await?;
-            (listener.local_addr()?, listener)
-        } else {
-            let listener = TcpListener::bind("0.0.0.0:0").await?;
-            let listener_address = listener.local_addr()?;
-            (listener_address, listener)
-        };
-        self.set_local_address(listener_address);
-        info!("Initializing listener for node ({:x})", self.name);
+        let listener = TcpListener::bind(&self.config.desired_address).await?;
+        let listener_address = listener.local_addr()?;
 
-        let node = self.clone();
+        self.set_local_address(listener_address);
+        info!("Initializing listener for node ({:x})", self.id);
+
+        let node_clone = self.clone();
         let listener_handle = task::spawn(async move {
             info!("Listening for nodes at {}", listener_address);
 
-            let bootnodes = node.config.bootnodes();
-
             loop {
                 match listener.accept().await {
                     Ok((stream, remote_address)) => {
                         info!("Got a connection request from {}", remote_address);
 
-                        // Wait a maximum timeout limit for a connection request.
-                        let timeout = match bootnodes.contains(&remote_address) {
-                            true => Duration::from_secs(crate::HANDSHAKE_BOOTNODE_TIMEOUT_SECS as u64),
-                            false => Duration::from_secs(crate::HANDSHAKE_PEER_TIMEOUT_SECS as u64),
-                        };
-                        let handshake_result = tokio::time::timeout(
-                            timeout,
-                            node.connection_request(listener_address, remote_address, stream),
-                        )
-                        .await;
+                        if !node_clone.can_connect() {
+                            metrics::increment_counter!(stats::CONNECTIONS_ALL_REJECTED);
+                            continue;
+                        }
 
-                        match handshake_result {
-                            Ok(Ok((mut writer, mut reader, remote_listener))) => {
-                                // Update the remote address to be the peer's listening address.
-                                let remote_address = writer.addr;
+                        let node = node_clone.clone();
+                        task::spawn(async move {
+                            // Wait a maximum timeout limit for a connection request.
+                            let handshake_result = tokio::time::timeout(
+                                Duration::from_secs(crate::HANDSHAKE_PEER_TIMEOUT_SECS as u64),
+                                node.connection_request(listener_address, remote_address, stream),
+                            )
+                            .await;
 
-                                // Create a channel dedicated to sending messages to the connection.
-                                let (sender, receiver) = channel(1024);
+                            match handshake_result {
+                                Ok(Ok((mut writer, mut reader, remote_listener))) => {
+                                    // Create a channel dedicated to sending messages to the connection.
+                                    let (sender, receiver) = channel(crate::OUTBOUND_CHANNEL_DEPTH);
 
-                                // Listen for inbound messages.
-                                let node_clone = node.clone();
-                                let peer_reading_task = tokio::spawn(async move {
-                                    node_clone.listen_for_inbound_messages(&mut reader).await;
-                                });
+                                    // Listen for inbound messages.
+                                    let node_clone = node.clone();
+                                    let peer_reading_task = tokio::spawn(async move {
+                                        node_clone.listen_for_inbound_messages(&mut reader).await;
+                                    });
 
-                                // Listen for outbound messages.
-                                let node_clone = node.clone();
-                                let peer_writing_task = tokio::spawn(async move {
-                                    node_clone.listen_for_outbound_messages(receiver, &mut writer).await;
-                                });
+                                    // Listen for outbound messages.
+                                    let node_clone = node.clone();
+                                    let peer_writing_task = tokio::spawn(async move {
+                                        node_clone.listen_for_outbound_messages(receiver, &mut writer).await;
+                                    });
 
-                                // Save the channel under the provided remote address.
-                                node.outbound.channels.write().insert(remote_address, sender);
+                                    // Save the channel under the provided remote address.
+                                    node.outbound.channels.write().insert(remote_listener, sender);
 
-                                // Finally, mark the peer as connected.
-                                node.peer_book.set_connected(remote_address, Some(remote_listener));
+                                    // Finally, mark the peer as connected.
+                                    node.peer_book.set_connected(remote_address, Some(remote_listener));
 
-                                trace!("Connected to {}", remote_address);
+                                    trace!("Connected to {} (listener: {})", remote_address, remote_listener);
 
-                                // Immediately send a ping to provide the peer with our block height.
-                                node.send_ping(remote_address).await;
+                                    // Immediately send a ping to provide the peer with our block height.
+                                    node.send_ping(remote_listener);
 
-                                if let Ok(ref peer) = node.peer_book.get_peer(remote_address) {
-                                    peer.register_task(peer_reading_task);
-                                    peer.register_task(peer_writing_task);
-                                } else {
-                                    // If the related peer is not found, it means it's already been dropped.
-                                    peer_reading_task.abort();
-                                    peer_writing_task.abort();
+                                    if let Ok(ref peer) = node.peer_book.get_peer(remote_listener) {
+                                        peer.register_task(peer_reading_task, true);
+                                        peer.register_task(peer_writing_task, false);
+                                    } else {
+                                        // If the related peer is not found, it means it's already been dropped.
+                                        peer_reading_task.abort();
+                                        peer_writing_task.abort();
+                                    }
+                                }
+                                Ok(Err(e)) => {
+                                    error!("Failed to accept a connection request: {}", e);
+                                    node.disconnect_from_peer(remote_address);
+                                    metrics::increment_counter!(stats::HANDSHAKES_FAILURES_RESP);
+                                }
+                                Err(_) => {
+                                    error!("Failed to accept a connection request: the handshake timed out");
+                                    node.disconnect_from_peer(remote_address);
+                                    metrics::increment_counter!(stats::HANDSHAKES_TIMEOUTS_RESP);
                                 }
                             }
-                            Ok(Err(e)) => {
-                                error!("Failed to accept a connection request: {}", e);
-                                let _ = node.disconnect_from_peer(remote_address);
-                            }
-                            Err(_) => {
-                                error!("Failed to accept a connection request: the handshake timed out");
-                                let _ = node.disconnect_from_peer(remote_address);
-                            }
-                        }
+                        });
+
+                        // add a tiny delay to avoid connecting above the limit
+                        tokio::time::sleep(Duration::from_millis(1)).await;
                     }
                     Err(e) => error!("Failed to accept a connection: {}", e),
                 }
+                metrics::increment_counter!(stats::CONNECTIONS_ALL_ACCEPTED);
             }
         });
 
@@ -183,9 +176,8 @@ impl<S: Storage + Send + Sync + 'static> Node<S> {
                     // Determine if we should send a disconnect message.
                     match disconnect_from_peer {
                         true => {
-                            // TODO (howardwu): Implement a handler so the node does not lose state of undetected disconnects.
                             warn!("Disconnecting from {} (unreliable)", reader.addr);
-                            let _ = self.disconnect_from_peer(reader.addr);
+                            self.disconnect_from_peer(reader.addr);
                             // The error has been handled and reported, we may now safely break.
                             break;
                         }
@@ -203,9 +195,7 @@ impl<S: Storage + Send + Sync + 'static> Node<S> {
                 // Handle Ping/Pong messages immediately in order not to skew latency calculation.
                 match &message.payload {
                     Payload::Ping(..) => {
-                        self.outbound
-                            .send_request(Message::new(Direction::Outbound(reader.addr), Payload::Pong))
-                            .await;
+                        self.send_request(Message::new(Direction::Outbound(reader.addr), Payload::Pong));
                     }
                     Payload::Pong => {
                         self.peer_book.received_pong(reader.addr);
@@ -214,106 +204,125 @@ impl<S: Storage + Send + Sync + 'static> Node<S> {
                 }
 
                 // Messages are queued in a single tokio MPSC receiver.
-                self.inbound.route(message).await
+                self.route(message)
             }
         }
     }
 
-    pub async fn process_incoming_messages(&self, receiver: &mut Receiver) -> Result<(), NetworkError> {
+    pub async fn process_incoming_messages(
+        &self,
+        receiver: &mut Receiver,
+        cache: &mut Cache,
+    ) -> Result<(), NetworkError> {
         let Message { direction, payload } = receiver.recv().await.ok_or(NetworkError::ReceiverFailedToParse)?;
 
+        metrics::decrement_gauge!(stats::QUEUES_INBOUND, 1.0);
+
         let source = if let Direction::Inbound(addr) = direction {
-            self.peer_book.update_last_seen(addr);
             addr
         } else {
             unreachable!("All messages processed sent to the inbound receiver are Inbound");
         };
 
+        self.peer_book.register_message(source);
+
+        // Check if the message hasn't already been processed recently if it's a `Block`.
+        // The node should also reject them while syncing, as it is bound to receive them later.
+        if matches!(payload, Payload::Block(..)) && (self.state() == State::Syncing || cache.contains(&payload)) {
+            return Ok(());
+        }
+
         match payload {
             Payload::Transaction(transaction) => {
+                metrics::increment_counter!(stats::INBOUND_TRANSACTIONS);
+
                 if let Some(ref sync) = self.sync() {
-                    sync.received_memory_pool_transaction(source, transaction).await?;
+                    sync.received_memory_pool_transaction(source, transaction)?;
                 }
             }
             Payload::Block(block) => {
+                metrics::increment_counter!(stats::INBOUND_BLOCKS);
+
                 if let Some(ref sync) = self.sync() {
-                    sync.received_block(source, block, true).await?;
+                    sync.received_block(source, block, true)?;
                 }
             }
             Payload::SyncBlock(block) => {
+                metrics::increment_counter!(stats::INBOUND_SYNCBLOCKS);
+
                 if let Some(ref sync) = self.sync() {
-                    sync.received_block(source, block, false).await?;
+                    sync.received_block(source, block, false)?;
 
                     // Update the peer and possibly finish the sync process.
                     if self.peer_book.got_sync_block(source) {
                         sync.finished_syncing_blocks();
-                    } else {
-                        // Since we confirmed that the block is a valid sync block
-                        // and we're expecting more blocks from the peer, we can set
-                        // the node's status to Syncing.
-                        sync.node().set_state(State::Syncing);
                     }
                 }
             }
             Payload::GetBlocks(hashes) => {
+                metrics::increment_counter!(stats::INBOUND_GETBLOCKS);
+
                 if let Some(ref sync) = self.sync() {
-                    sync.received_get_blocks(source, hashes).await?;
+                    sync.received_get_blocks(source, hashes)?;
                 }
             }
             Payload::GetMemoryPool => {
+                metrics::increment_counter!(stats::INBOUND_GETMEMORYPOOL);
+
                 if let Some(ref sync) = self.sync() {
-                    sync.received_get_memory_pool(source).await?;
+                    sync.received_get_memory_pool(source);
                 }
             }
             Payload::MemoryPool(mempool) => {
+                metrics::increment_counter!(stats::INBOUND_MEMORYPOOL);
+
                 if let Some(ref sync) = self.sync() {
                     sync.received_memory_pool(mempool)?;
                 }
             }
             Payload::GetSync(getsync) => {
+                metrics::increment_counter!(stats::INBOUND_GETSYNC);
+
                 if let Some(ref sync) = self.sync() {
-                    sync.received_get_sync(source, getsync).await?;
+                    sync.received_get_sync(source, getsync)?;
                 }
             }
             Payload::Sync(sync) => {
+                metrics::increment_counter!(stats::INBOUND_SYNCS);
+
                 if let Some(ref sync_handler) = self.sync() {
                     if sync.is_empty() {
-                        trace!("{} doesn't have sync blocks to share", source);
+                        // An empty `Sync` is unexpected, as `GetSync` requests are only
+                        // sent to peers that declare a greater block height.
+                        self.peer_book.register_failure(source);
+                        warn!("{} doesn't have sync blocks to share", source);
                     } else if self.peer_book.expecting_sync_blocks(source, sync.len()) {
                         trace!("Received {} sync block hashes from {}", sync.len(), source);
-                        sync_handler.received_sync(source, sync).await;
+                        sync_handler.received_sync(source, sync);
                     }
                 }
             }
             Payload::GetPeers => {
-                self.send_peers(source).await;
+                metrics::increment_counter!(stats::INBOUND_GETPEERS);
+
+                self.send_peers(source);
             }
             Payload::Peers(peers) => {
+                metrics::increment_counter!(stats::INBOUND_PEERS);
+
                 self.process_inbound_peers(peers);
             }
             Payload::Ping(block_height) => {
-                self.peer_book.received_ping(source, block_height);
+                metrics::increment_counter!(stats::INBOUND_PINGS);
 
-                // TODO (howardwu): Delete me after stabilizing new sync logic for blocks.
-                // if let Some(ref sync) = self.sync() {
-                //     if block_height > sync.current_block_height() + 1 {
-                //         // If the node is syncing, check if that sync attempt hasn't expired.
-                //         if !sync.is_syncing_blocks() || sync.has_block_sync_expired() {
-                //             // Cancel any possibly ongoing sync attempts.
-                //             self.set_state(State::Idle);
-                //             self.peer_book.cancel_any_unfinished_syncing();
-                //
-                //             // Begin a new sync attempt.
-                //             sync.register_block_sync_attempt(source);
-                //             sync.update_blocks(source).await;
-                //         }
-                //     }
-                // }
+                self.peer_book.received_ping(source, block_height);
             }
             Payload::Pong => {
+                metrics::increment_counter!(stats::INBOUND_PONGS);
                 // Skip as this case is already handled with priority in Inbound::listen_for_messages
             }
             Payload::Unknown => {
+                metrics::increment_counter!(stats::INBOUND_UNKNOWN);
                 warn!("Unknown payload received; this could indicate that the client you're using is out-of-date");
             }
         }
@@ -357,7 +366,8 @@ impl<S: Storage + Send + Sync + 'static> Node<S> {
         trace!("received e (XX handshake part 1/3) from {}", remote_address);
 
         // -> e, ee, s, es
-        let own_version = Version::serialize(&Version::new(1u64, listener_address.port())).unwrap(); // TODO (raychu86): Establish a formal node version.
+        let own_version =
+            Version::serialize(&Version::new(crate::PROTOCOL_VERSION, listener_address.port(), self.id)).unwrap();
         let len = noise.write_message(&own_version, &mut buffer)?;
         writer.write_all(&[len as u8]).await?;
         writer.write_all(&buffer[..len]).await?;
@@ -374,6 +384,15 @@ impl<S: Storage + Send + Sync + 'static> Node<S> {
         let peer_version = Version::deserialize(&buffer[..len])?;
         trace!("received s, se, psk (XX handshake part 3/3) from {}", remote_address);
 
+        if peer_version.node_id == self.id {
+            return Err(NetworkError::SelfConnectAttempt);
+        }
+        if peer_version.version != crate::PROTOCOL_VERSION {
+            return Err(NetworkError::InvalidHandshake);
+        }
+
+        metrics::increment_counter!(stats::HANDSHAKES_SUCCESSES_RESP);
+
         // the remote listening address
         let remote_listener = SocketAddr::from((remote_address.ip(), peer_version.listening_port));
 
@@ -383,4 +402,21 @@ impl<S: Storage + Send + Sync + 'static> Node<S> {
 
         Ok((writer, reader, remote_listener))
     }
+
+    #[inline]
+    pub(crate) fn route(&self, response: Message) {
+        match self.inbound.sender.try_send(response) {
+            Err(TrySendError::Full(msg)) => {
+                metrics::increment_counter!(stats::INBOUND_ALL_FAILURES);
+                error!("Failed to route a {}: the inbound channel is full", msg);
+            }
+            Err(TrySendError::Closed(msg)) => {
+                // TODO: this shouldn't happen, but is critical if it does
+                error!("Failed to route a {}: the inbound channel is closed", msg);
+            }
+            Ok(_) => {
+                metrics::increment_gauge!(stats::QUEUES_INBOUND, 1.0);
+            }
+        }
+    }
 }
